#!/bin/bash

echo "Step1: Installing jq package"
sudo apt-get update -y
sudo apt-get install -y jq
sudo touch /etc/netq/sim/sim-data/netq-switchd-simulator

echo "Step2: Enable AR simulation"
sudo touch /etc/netq/sim/sim-data/enable_arconfig.json

roce_mode="Lossless"

echo "Step3: Read the interfaces from NVUE"
if [ -z "$1" ]; then
    echo "Picking up configurations from NVUE - Interface, AR, RoCE"
    ifnames=$(nv show interface -o json | jq 'to_entries[] | select(.value.type == "swp") | .key')
    ifnames_arr=$(echo $ifnames | jq -R 'split(" ") | map(fromjson)')
    roce_mode=$(nv show qos roce  -o json | jq '.mode = (.mode | ascii_upcase[0:1] + .[1:])'| jq -r .mode)
else
    echo "Pick up interfaces from predefined list template"
    ifnames_arr='["swp10s0", "swp10s1", "swp11s0", "swp11s1", "swp12s0", "swp12s1", "swp13s0", "swp13s1", "swp14s0", "swp14s1", "swp15s0", "swp15s1", "swp16s0", "swp16s1", "swp17s0", "swp17s1", "swp18s0", "swp18s1", "swp19s0", "swp19s1", "swp1s0", "swp1s1", "swp20s0", "swp20s1", "swp21s0", "swp21s1", "swp22s0", "swp22s1", "swp23s0", "swp23s1", "swp24s0", "swp24s1", "swp25s0", "swp25s1", "swp26s0", "swp26s1", "swp27s0", "swp27s1", "swp28s0", "swp28s1", "swp29s0", "swp29s1", "swp2s0", "swp2s1", "swp30s0", "swp30s1", "swp31s0", "swp31s1", "swp32s0", "swp32s1", "swp33s0", "swp33s1", "swp34s0", "swp34s1", "swp35s0", "swp35s1", "swp36s0", "swp36s1", "swp37s0", "swp37s1", "swp38s0", "swp38s1", "swp39s0", "swp39s1", "swp3s0", "swp3s1", "swp40s0", "swp40s1", "swp41s0", "swp41s1", "swp42s0", "swp42s1", "swp43s0", "swp43s1", "swp44s0", "swp44s1", "swp45s0", "swp45s1", "swp46s0", "swp46s1", "swp47s0", "swp47s1", "swp48s0", "swp48s1", "swp49s0", "swp49s1", "swp4s0", "swp4s1", "swp50s0", "swp50s1", "swp51s0", "swp51s1", "swp52s0", "swp52s1", "swp53s0", "swp53s1", "swp54s0", "swp54s1", "swp55s0", "swp55s1", "swp56s0", "swp56s1", "swp57s0", "swp57s1", "swp58s0", "swp58s1", "swp59s0", "swp59s1", "swp5s0", "swp5s1", "swp60s0", "swp60s1", "swp61s0", "swp61s1", "swp62s0", "swp62s1", "swp63s0", "swp63s1", "swp64s0", "swp64s1", "swp6s0", "swp6s1", "swp7s0", "swp7s1", "swp8s0", "swp8s1", "swp9s0", "swp9s1"]'
   echo "Assume roce mode as lossless"
   roce_mode="Lossless"
fi 

echo "Step4: Simulate interfaces directory path"
sudo rm -rf "/etc/netq/sim/sim-data/interfaces/"
for dir_name in $(echo $ifnames_arr | jq -r '.[]'); do
    # Create a directory with the name of the current element
    sudo mkdir -p "/etc/netq/sim/sim-data/interfaces/sys/devices/virtual/net/$dir_name/"
    sudo touch "/etc/netq/sim/sim-data/interfaces/sys/devices/virtual/net/$dir_name/dummy"
done

echo "Step5: Simulate output of /usr/lib/cumulus/mlxcmd --json ports info"
sudo jq -n --argjson ifnames "$ifnames_arr" '
  reduce range(0; $ifnames | length) as $i (
    {};
    . + { (65537 + $i | tostring): {
      "ifname": $ifnames[$i],
      "status": "Up",
      "speed": 0,
      "mtu": 9238,
      "fec": "Unknown"
    }}
  )
' | sudo tee /etc/netq/sim/sim-data/port_info.json > /dev/null

echo "Step6: Simulate output of /usr/lib/cumulus/mlxcmd --json roce status --system"
sudo jq -n --argjson ifnames "$(echo $ifnames_arr)" '
  {
    pools: 
      ($ifnames | map(
        {
          (.): {
        "2": {
          "cell_unit_size": 192,
          "name": "lossy-default-ingress",
          "current-usage-bytes": 0,
          "cells": 102940,
          "bytes": 14823360,
          "max-usage": 0,
          "max-usage-bytes": 0,
          "current-usage": 0,
          "mode": "DYNAMIC",
          "infinite": 0,
          "type": "User",
          "buff-type": "data buffer",
          "dir": "INGRESS"
        },
        "3": {
          "cell_unit_size": 192,
          "name": "roce-reserved-ingress",
          "current-usage-bytes": 0,
          "cells": 102940,
          "bytes": 14823360,
          "max-usage": 0,
          "max-usage-bytes": 0,
          "current-usage": 0,
          "mode": "DYNAMIC",
          "infinite": 0,
          "type": "User",
          "buff-type": "data buffer",
          "dir": "INGRESS"
        },
        "13": {
          "cell_unit_size": 192,
          "name": "lossy-default-egress",
          "current-usage-bytes": 0,
          "cells": 102940,
          "bytes": 14823360,
          "max-usage": 0,
          "max-usage-bytes": 0,
          "current-usage": 0,
          "mode": "DYNAMIC",
          "infinite": 0,
          "type": "User",
          "buff-type": "data buffer",
          "dir": "EGRESS"
        },
        "14": {
          "cell_unit_size": 192,
          "name": "roce-reserved-egress",
          "current-usage-bytes": 0,
          "cells": 102940,
          "bytes": 14823360,
          "max-usage": 0,
          "max-usage-bytes": 0,
          "current-usage": 0,
          "mode": "DYNAMIC",
          "infinite": 0,
          "type": "User",
          "buff-type": "data buffer",
          "dir": "EGRESS"
        }
        }
      }
      ))
  }
' | sudo tee /etc/netq/sim/sim-data/roce_status.json > /dev/null

echo "Step7: Simulate output of /usr/lib/cumulus/mlxcmd --json roce status"
sudo jq -n --argjson ifnames "$ifnames_arr" --arg roce_mode "$roce_mode" '
  reduce range(0; $ifnames | length) as $i (
  [];
  . + [{
    "port_id": (65537 + $i),
    "roce_mode": $roce_mode,
    "congestion-control": {
      "ecn_tcs": [
        0,
        3
      ],
      "mode": "absolute",
      "red_enabled": 0,
      "ecn_enabled": 1,
      "max_threshold": "1476.0 KB",
      "max_threshold_bytes": 1511424,
      "min_threshold": "156.0 KB",
      "min_threshold_bytes": 159744
    },
    "trust": {
      "dscp": "yes",
      "pcpdei": "yes",
      "trust_port": "no"
    },
    "roce_dscp": {
      "dscp": 26,
      "sp": 3,
      "color": 0
    },
    "cnp_dscp": {
      "dscp": 48,
      "sp": 6,
      "color": 0
    },
    "roce_pcp": {
      "pcp": 3,
      "dei": 1,
      "sp": 3,
      "color": 0
    },
    "cnp_pcp": {
      "pcp": 6,
      "dei": 1,
      "sp": 6,
      "color": 0
    },
    "roce_pg_map": {
      "sp": 3,
      "pg": 1
    },
    "cnp_pg_map": {
      "sp": 6,
      "pg": 0
    },
    "roce_tc_map": {
      "tc": 3,
      "sp": 3
    },
    "cnp_tc_map": {
      "tc": 6,
      "sp": 6
    },
    "pfc_priority": 3,
    "pfc_enabled": {
      "tx_enabled": "yes",
      "rx_enabled": "yes"
    },
    "roce_sched_map": {
      "level": 2,
      "index": 3,
      "parent_index": 0,
      "min_shaper_enable": 1,
      "min_shaper_rate": 0,
      "max_shaper_enable": 1,
      "max_shaper_rate": 2147483647,
      "shaper_mode": "bytes",
      "dwrr_enable": 1,
      "dwrr_mode": "dwrr",
      "dwrr_weight": 50
    },
    "cnp_sched_map": {
      "level": 2,
      "index": 6,
      "parent_index": 0,
      "min_shaper_enable": 1,
      "min_shaper_rate": 0,
      "max_shaper_enable": 1,
      "max_shaper_rate": 2147483647,
      "shaper_mode": "bytes",
      "dwrr_enable": 1,
      "dwrr_mode": "strict priority",
      "dwrr_weight": 0
    },
    "port": $ifnames[$i]
  }]
  )
' |  sudo tee /etc/netq/sim/sim-data/roce_config.json > /dev/null

echo "Step8: Simulate output of /usr/lib/cumulus/mlxcmd --json roce counters"
length=$(echo $ifnames_arr | jq 'length')
rx_roce_pg_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pg_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_no_buffer_discard=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_pg_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_pg_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_no_buffer_discard=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pfc_pause_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pfc_pause_duration=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_pfc_pause_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_pfc_pause_duration=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_ecn_marked_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_tc_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_tc_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_unicast_no_buffer_discard=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_tc_packets=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_tc_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_unicast_no_buffer_discard=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_buffer_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_buffer_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_buffer_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_buffer_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_buffer_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_buffer_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_buffer_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_buffer_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_pg_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_pg_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_pg_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_cnp_pg_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pg_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pg_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pg_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
rx_roce_pg_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_buffer_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_buffer_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_buffer_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_buffer_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_buffer_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_buffer_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_buffer_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_buffer_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_tc_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_tc_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_tc_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_roce_tc_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_tc_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .),
tx_cnp_tc_max_usage_cells=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_tc_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)
tx_cnp_tc_max_usage_bytes=$(shuf -i 1000-100000 -n $length | jq -s .)

sudo jq -n --argjson ifnames "$ifnames_arr" --argjson rx_roce_pg_packets "$rx_roce_pg_packets" --argjson rx_roce_pg_bytes "$rx_roce_pg_bytes" --argjson rx_roce_no_buffer_discard "$rx_roce_no_buffer_discard" --argjson rx_cnp_pg_packets "$rx_cnp_pg_packets" --argjson rx_cnp_pg_bytes "$rx_cnp_pg_bytes" --argjson rx_cnp_no_buffer_discard "$rx_cnp_no_buffer_discard" --argjson rx_roce_pfc_pause_packets "$rx_roce_pfc_pause_packets" --argjson rx_roce_pfc_pause_duration "$rx_roce_pfc_pause_duration"  --argjson tx_roce_pfc_pause_packets "$tx_roce_pfc_pause_packets" --argjson tx_roce_pfc_pause_duration "$tx_roce_pfc_pause_duration" --argjson tx_ecn_marked_packets "$tx_ecn_marked_packets" --argjson tx_roce_tc_packets "$tx_roce_tc_packets" --argjson tx_roce_tc_bytes "$tx_roce_tc_bytes" --argjson tx_roce_unicast_no_buffer_discard "$tx_roce_unicast_no_buffer_discard" '
  reduce range(0; $ifnames | length) as $i (
  [];
  . + [{
    "port_id": (65537 + $i),
    "rx_roce_pg_packets": $rx_roce_pg_packets[$i],
    "rx_roce_pg_bytes": $rx_roce_pg_bytes[$i],
    "rx_roce_no_buffer_discard": $rx_roce_no_buffer_discard[$i],
    "rx_cnp_pg_packets": $rx_cnp_pg_packets[$i],
    "rx_cnp_pg_bytes": $rx_cnp_pg_bytes[$i],
    "rx_cnp_no_buffer_discard": $rx_cnp_no_buffer_discard[$i],
    "rx_roce_pfc_pause_packets": $rx_roce_pfc_pause_packets[$i],
    "rx_roce_pfc_pause_duration": $rx_roce_pfc_pause_duration[$i],
    "tx_roce_pfc_pause_packets": $tx_roce_pfc_pause_packets[$i],
    "tx_roce_pfc_pause_duration": $tx_roce_pfc_pause_duration[$i],
    "tx_ecn_marked_packets": $tx_ecn_marked_packets[$i],
    "tx_roce_tc_packets": $tx_roce_tc_packets[$i],
    "tx_roce_tc_bytes": $tx_roce_tc_bytes[$i],
    "tx_roce_unicast_no_buffer_discard": $tx_roce_unicast_no_buffer_discard[$i],
    "tx_cnp_tc_packets": 0,
    "tx_cnp_tc_bytes": 0,
    "tx_cnp_unicast_no_buffer_discard": 0,
    "rx_cnp_buffer_usage_cells": 0,
    "rx_cnp_buffer_max_usage_cells": 0,
    "rx_cnp_buffer_usage_bytes": 0,
    "rx_cnp_buffer_max_usage_bytes": 0,
    "rx_roce_buffer_usage_cells": 0,
    "rx_roce_buffer_max_usage_cells": 0,
    "rx_roce_buffer_usage_bytes": 0,
    "rx_roce_buffer_max_usage_bytes": 0,
    "rx_cnp_pg_usage_cells": 0,
    "rx_cnp_pg_max_usage_cells": 0,
    "rx_cnp_pg_usage_bytes": 0,
    "rx_cnp_pg_max_usage_bytes": 0,
    "rx_roce_pg_usage_cells": 0,
    "rx_roce_pg_max_usage_cells": 0,
    "rx_roce_pg_usage_bytes": 0,
    "rx_roce_pg_max_usage_bytes": 0,
    "tx_cnp_buffer_usage_cells": 0,
    "tx_cnp_buffer_max_usage_cells": 0,
    "tx_cnp_buffer_usage_bytes": 0,
    "tx_cnp_buffer_max_usage_bytes": 0,
    "tx_roce_buffer_usage_cells": 0,
    "tx_roce_buffer_max_usage_cells": 0,
    "tx_roce_buffer_usage_bytes": 0,
    "tx_roce_buffer_max_usage_bytes": 0,
    "tx_roce_tc_usage_cells": 0,
    "tx_roce_tc_max_usage_cells": 0,
    "tx_roce_tc_usage_bytes": 0,
    "tx_roce_tc_max_usage_bytes": 0,
    "tx_cnp_tc_usage_cells": 0,
    "tx_cnp_tc_max_usage_cells": 0,
    "tx_cnp_tc_usage_bytes": 0,
    "tx_cnp_tc_max_usage_bytes": 0,
    "cell_unit_size": 192,
    "port": $ifnames[$i]
  }]
  )
' | sudo tee /etc/netq/sim/sim-data/roce_counters.json > /dev/null
